"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 7d0ec3402dc0

from __future__ import annotations
from .tempotraceattributeboolvalue import (
    TempoTraceAttributeBoolValue,
    TempoTraceAttributeBoolValueTypedDict,
)
from .tempotraceattributeintvalue import (
    TempoTraceAttributeIntValue,
    TempoTraceAttributeIntValueTypedDict,
)
from .tempotraceattributestringvalue import (
    TempoTraceAttributeStringValue,
    TempoTraceAttributeStringValueTypedDict,
)
from mistralai.client.types import BaseModel
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


TempoTraceAttributeValueTypedDict = TypeAliasType(
    "TempoTraceAttributeValueTypedDict",
    Union[
        TempoTraceAttributeStringValueTypedDict,
        TempoTraceAttributeIntValueTypedDict,
        TempoTraceAttributeBoolValueTypedDict,
    ],
)
r"""The value of the attribute"""


TempoTraceAttributeValue = TypeAliasType(
    "TempoTraceAttributeValue",
    Union[
        TempoTraceAttributeStringValue,
        TempoTraceAttributeIntValue,
        TempoTraceAttributeBoolValue,
    ],
)
r"""The value of the attribute"""


class TempoTraceAttributeTypedDict(TypedDict):
    key: str
    r"""The key of the attribute"""
    value: TempoTraceAttributeValueTypedDict
    r"""The value of the attribute"""


class TempoTraceAttribute(BaseModel):
    key: str
    r"""The key of the attribute"""

    value: TempoTraceAttributeValue
    r"""The value of the attribute"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: c4462a05fb08

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


ResponseFormats = Union[
    Literal[
        "text",
        "json_object",
        "json_schema",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .deltamessage import DeltaMessage, DeltaMessageTypedDict
from mistralai.gcp.client.types import (
    BaseModel,
    Nullable,
    UNSET_SENTINEL,
    UnrecognizedStr,
)
from pydantic import model_serializer
from typing import Literal, Union
from typing_extensions import TypedDict


CompletionResponseStreamChoiceFinishReason = Union[
    Literal[
        "stop",
        "length",
        "error",
        "tool_calls",
    ],
    UnrecognizedStr,
]


class CompletionResponseStreamChoiceTypedDict(TypedDict):
    index: int
    delta: DeltaMessageTypedDict
    finish_reason: Nullable[CompletionResponseStreamChoiceFinishReason]


class CompletionResponseStreamChoice(BaseModel):
    index: int

    delta: DeltaMessage

    finish_reason: Nullable[CompletionResponseStreamChoiceFinishReason]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from typing import Literal


ToolChoiceEnum = Literal[
    "auto",
    "none",
    "any",
    "required",
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from mistralai.azure.client.errors import MistralAzureError
from mistralai.azure.client.models import validationerror as models_validationerror
from mistralai.azure.client.types import BaseModel
from typing import List, Optional


class HTTPValidationErrorData(BaseModel):
    detail: Optional[List[models_validationerror.ValidationError]] = None


@dataclass(unsafe_hash=True)
class HTTPValidationError(MistralAzureError):
    data: HTTPValidationErrorData = field(hash=False)

    def __init__(
        self,
        data: HTTPValidationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 00693c7eec60

from __future__ import annotations
from .basemodelcard import BaseModelCard, BaseModelCardTypedDict
from .ftmodelcard import FTModelCard, FTModelCardTypedDict
from functools import partial
from mistralai.client.types import BaseModel, UNSET_SENTINEL
from mistralai.client.utils.unions import parse_open_union
from pydantic import ConfigDict, model_serializer
from pydantic.functional_validators import BeforeValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ModelListDataTypedDict = TypeAliasType(
    "ModelListDataTypedDict", Union[BaseModelCardTypedDict, FTModelCardTypedDict]
)


class UnknownModelListData(BaseModel):
    r"""A ModelListData variant the SDK doesn't recognize. Preserves the raw payload."""

    type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_MODEL_LIST_DATA_VARIANTS: dict[str, Any] = {
    "base": BaseModelCard,
    "fine-tuned": FTModelCard,
}


ModelListData = Annotated[
    Union[BaseModelCard, FTModelCard, UnknownModelListData],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="type",
            variants=_MODEL_LIST_DATA_VARIANTS,
            unknown_cls=UnknownModelListData,
            union_name="ModelListData",
        )
    ),
]


class ModelListTypedDict(TypedDict):
    object: NotRequired[str]
    data: NotRequired[List[ModelListDataTypedDict]]


class ModelList(BaseModel):
    object: Optional[str] = "list"

    data: Optional[List[ModelListData]] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["object", "data"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: d23713342634

from typing import Any

from pydantic import BaseModel, TypeAdapter


def parse_open_union(
    v: Any,
    *,
    disc_key: str,
    variants: dict[str, Any],
    unknown_cls: type,
    union_name: str,
) -> Any:
    """Parse an open discriminated union value with forward-compatibility.

    Known discriminator values are dispatched to their variant types.
    Unknown discriminator values produce an instance of the fallback class,
    preserving the raw payload for inspection.
    """
    if isinstance(v, BaseModel):
        return v
    if not isinstance(v, dict) or disc_key not in v:
        raise ValueError(f"{union_name}: expected object with '{disc_key}' field")
    disc = v[disc_key]
    variant_cls = variants.get(disc)
    if variant_cls is not None:
        if isinstance(variant_cls, type) and issubclass(variant_cls, BaseModel):
            return variant_cls.model_validate(v)
        return TypeAdapter(variant_cls).validate_python(v)
    return unknown_cls(raw=v)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: cc807b30de19

import importlib.metadata

__title__: str = "mistralai"
__version__: str = "2.2.0rc3"
__openapi_doc_version__: str = "1.0.0"
__gen_version__: str = "2.862.0"
__user_agent__: str = "speakeasy-sdk/python 2.2.0rc3 2.862.0 1.0.0 mistralai"

try:
    if __package__ is not None:
        __version__ = importlib.metadata.version(__package__)
except importlib.metadata.PackageNotFoundError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 118e05dbfbbd

from __future__ import annotations
from .classifierfinetuningjob import (
    ClassifierFineTuningJob,
    ClassifierFineTuningJobTypedDict,
)
from .completionfinetuningjob import (
    CompletionFineTuningJob,
    CompletionFineTuningJobTypedDict,
)
from functools import partial
from mistralai.client.types import BaseModel, UNSET_SENTINEL
from mistralai.client.utils import validate_const
from mistralai.client.utils.unions import parse_open_union
import pydantic
from pydantic import ConfigDict, model_serializer
from pydantic.functional_validators import AfterValidator, BeforeValidator
from typing import Any, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


ListFineTuningJobsResponseDataTypedDict = TypeAliasType(
    "ListFineTuningJobsResponseDataTypedDict",
    Union[ClassifierFineTuningJobTypedDict, CompletionFineTuningJobTypedDict],
)


class UnknownListFineTuningJobsResponseData(BaseModel):
    r"""A ListFineTuningJobsResponseData variant the SDK doesn't recognize. Preserves the raw payload."""

    job_type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_LIST_FINE_TUNING_JOBS_RESPONSE_DATA_VARIANTS: dict[str, Any] = {
    "classifier": ClassifierFineTuningJob,
    "completion": CompletionFineTuningJob,
}


ListFineTuningJobsResponseData = Annotated[
    Union[
        ClassifierFineTuningJob,
        CompletionFineTuningJob,
        UnknownListFineTuningJobsResponseData,
    ],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="job_type",
            variants=_LIST_FINE_TUNING_JOBS_RESPONSE_DATA_VARIANTS,
            unknown_cls=UnknownListFineTuningJobsResponseData,
            union_name="ListFineTuningJobsResponseData",
        )
    ),
]


class ListFineTuningJobsResponseTypedDict(TypedDict):
    total: int
    data: NotRequired[List[ListFineTuningJobsResponseDataTypedDict]]
    object: Literal["list"]


class ListFineTuningJobsResponse(BaseModel):
    total: int

    data: Optional[List[ListFineTuningJobsResponseData]] = None

    object: Annotated[
        Annotated[Optional[Literal["list"]], AfterValidator(validate_const("list"))],
        pydantic.Field(alias="object"),
    ] = "list"

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["data", "object"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    ListFineTuningJobsResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 9c9947e768d3

from __future__ import annotations
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from typing_extensions import Annotated, TypedDict


class AgentsAPIV1AgentsDeleteAliasRequestTypedDict(TypedDict):
    agent_id: str
    alias: str


class AgentsAPIV1AgentsDeleteAliasRequest(BaseModel):
    agent_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    alias: Annotated[
        str, FieldMetadata(query=QueryParamMetadata(style="form", explode=True))
    ]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ea15ddd64402

from __future__ import annotations
from datetime import datetime
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class DeploymentResponseTypedDict(TypedDict):
    id: str
    r"""Unique identifier of the deployment"""
    name: str
    r"""Deployment name"""
    is_active: bool
    r"""Whether at least one worker is currently live"""
    created_at: datetime
    r"""When the deployment was first registered"""
    updated_at: datetime
    r"""When the deployment was last updated"""


class DeploymentResponse(BaseModel):
    id: str
    r"""Unique identifier of the deployment"""

    name: str
    r"""Deployment name"""

    is_active: bool
    r"""Whether at least one worker is currently live"""

    created_at: datetime
    r"""When the deployment was first registered"""

    updated_at: datetime
    r"""When the deployment was last updated"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6ea58a356f77

from __future__ import annotations
from .schedulecalendar import ScheduleCalendar, ScheduleCalendarTypedDict
from .scheduleinterval import ScheduleInterval, ScheduleIntervalTypedDict
from .schedulepolicy import SchedulePolicy, SchedulePolicyTypedDict
from datetime import datetime
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Any, List, Optional
from typing_extensions import NotRequired, TypedDict


class ScheduleDefinitionTypedDict(TypedDict):
    r"""Specification of the times scheduled actions may occur.

    The times are the union of :py:attr:`calendars`, :py:attr:`intervals`, and
    :py:attr:`cron_expressions` excluding anything in :py:attr:`skip`.

    Used for input where schedule_id is optional (can be provided or auto-generated).
    """

    input: Any
    r"""Input to provide to the workflow when starting it."""
    calendars: NotRequired[List[ScheduleCalendarTypedDict]]
    r"""Calendar-based specification of times."""
    intervals: NotRequired[List[ScheduleIntervalTypedDict]]
    r"""Interval-based specification of times."""
    cron_expressions: NotRequired[List[str]]
    r"""Cron-based specification of times."""
    skip: NotRequired[List[ScheduleCalendarTypedDict]]
    r"""Set of calendar times to skip."""
    start_at: NotRequired[Nullable[datetime]]
    r"""Time after which the first action may be run."""
    end_at: NotRequired[Nullable[datetime]]
    r"""Time after which no more actions will be run."""
    jitter: NotRequired[Nullable[str]]
    r"""Jitter to apply each action.

    An action's scheduled time will be incremented by a random value between 0
    and this value if present (but not past the next schedule).

    """
    time_zone_name: NotRequired[Nullable[str]]
    r"""IANA time zone name, for example ``US/Central``."""
    policy: NotRequired[SchedulePolicyTypedDict]
    schedule_id: NotRequired[Nullable[str]]
    r"""Unique identifier for the schedule."""


class ScheduleDefinition(BaseModel):
    r"""Specification of the times scheduled actions may occur.

    The times are the union of :py:attr:`calendars`, :py:attr:`intervals`, and
    :py:attr:`cron_expressions` excluding anything in :py:attr:`skip`.

    Used for input where schedule_id is optional (can be provided or auto-generated).
    """

    input: Any
    r"""Input to provide to the workflow when starting it."""

    calendars: Optional[List[ScheduleCalendar]] = None
    r"""Calendar-based specification of times."""

    intervals: Optional[List[ScheduleInterval]] = None
    r"""Interval-based specification of times."""

    cron_expressions: Optional[List[str]] = None
    r"""Cron-based specification of times."""

    skip: Optional[List[ScheduleCalendar]] = None
    r"""Set of calendar times to skip."""

    start_at: OptionalNullable[datetime] = UNSET
    r"""Time after which the first action may be run."""

    end_at: OptionalNullable[datetime] = UNSET
    r"""Time after which no more actions will be run."""

    jitter: OptionalNullable[str] = UNSET
    r"""Jitter to apply each action.

    An action's scheduled time will be incremented by a random value between 0
    and this value if present (but not past the next schedule).

    """

    time_zone_name: OptionalNullable[str] = UNSET
    r"""IANA time zone name, for example ``US/Central``."""

    policy: Optional[SchedulePolicy] = None

    schedule_id: OptionalNullable[str] = UNSET
    r"""Unique identifier for the schedule."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "calendars",
                "intervals",
                "cron_expressions",
                "skip",
                "start_at",
                "end_at",
                "jitter",
                "time_zone_name",
                "policy",
                "schedule_id",
            ]
        )
        nullable_fields = set(
            ["start_at", "end_at", "jitter", "time_zone_name", "schedule_id"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b74b57603a4c

from __future__ import annotations
from .basetaskstatus import BaseTaskStatus
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class FetchCampaignStatusResponseTypedDict(TypedDict):
    status: BaseTaskStatus


class FetchCampaignStatusResponse(BaseModel):
    status: BaseTaskStatus



# MAKE SURE YOU UPDATE THE COPIES OF THIS FILES IN THE PROVIDERS'S PACKAGES WHEN YOU MAKE CHANGES HERE
from typing import Union

import httpx

from .types import BeforeRequestContext, BeforeRequestHook

PREFIX = "mistral-client-python/"

class CustomUserAgentHook(BeforeRequestHook):
    def before_request(
        self, hook_ctx: BeforeRequestContext, request: httpx.Request
    ) -> Union[httpx.Request, Exception]:
        current = request.headers["user-agent"]
        if current.startswith(PREFIX):
            return request
        
        request.headers["user-agent"] = (
            PREFIX + current.split(" ")[1]
        )

        return request



from typing import Any, TypeVar, cast

from pydantic import BaseModel
from mistralai.client.models import ResponseFormatTypedDict
from ._pydantic_helper import rec_strict_json_schema

CustomPydanticModel = TypeVar("CustomPydanticModel", bound=BaseModel)


def response_format_from_pydantic_model(
    model: type[CustomPydanticModel],
) -> ResponseFormatTypedDict:
    """Generate a strict JSON schema response format from a pydantic model.

    Returns a TypedDict compatible with both the main SDK's and Azure SDK's
    ResponseFormat / ResponseFormatTypedDict.
    """
    model_schema = rec_strict_json_schema(model.model_json_schema())
    return cast(
        ResponseFormatTypedDict,
        {
            "type": "json_schema",
            "json_schema": {
                "name": model.__name__,
                "schema": model_schema,
                "strict": True,
            },
        },
    )


def pydantic_model_from_json(
    json_data: dict[str, Any],
    pydantic_model: type[CustomPydanticModel],
) -> CustomPydanticModel:
    """Parse a JSON schema into a pydantic model."""
    return pydantic_model.model_validate(json_data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 1813f339625b

from .basesdk import BaseSDK
from .sdkconfiguration import SDKConfiguration
from mistralai.client import errors, models, utils
from mistralai.client._hooks import HookContext
from mistralai.client.fields import Fields
from mistralai.client.types import OptionalNullable, UNSET
from mistralai.client.utils import get_security_from_env
from mistralai.client.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class ChatCompletionEvents(BaseSDK):
    fields: Fields

    def __init__(
        self, sdk_config: SDKConfiguration, parent_ref: Optional[object] = None
    ) -> None:
        BaseSDK.__init__(self, sdk_config, parent_ref=parent_ref)
        self.sdk_configuration = sdk_config
        self._init_sdks()

    def _init_sdks(self):
        self.fields = Fields(self.sdk_configuration, parent_ref=self.parent_ref)

    def search(
        self,
        *,
        search_params: Union[models.FilterPayload, models.FilterPayloadTypedDict],
        page_size: Optional[int] = 50,
        cursor: OptionalNullable[str] = UNSET,
        extra_fields: OptionalNullable[List[str]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SearchChatCompletionEventsResponse:
        r"""Get Chat Completion Events

        :param search_params:
        :param page_size:
        :param cursor:
        :param extra_fields:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetChatCompletionEventsV1ObservabilityChatCompletionEventsSearchPostRequest(
            page_size=page_size,
            cursor=cursor,
            search_chat_completion_events_request=models.SearchChatCompletionEventsRequest(
                search_params=utils.get_pydantic_model(
                    search_params, models.FilterPayload
                ),
                extra_fields=extra_fields,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/observability/chat-completion-events/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.search_chat_completion_events_request,
                False,
                False,
                "json",
                models.SearchChatCompletionEventsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_chat_completion_events_v1_observability_chat_completion_events_search_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SearchChatCompletionEventsResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def search_async(
        self,
        *,
        search_params: Union[models.FilterPayload, models.FilterPayloadTypedDict],
        page_size: Optional[int] = 50,
        cursor: OptionalNullable[str] = UNSET,
        extra_fields: OptionalNullable[List[str]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SearchChatCompletionEventsResponse:
        r"""Get Chat Completion Events

        :param search_params:
        :param page_size:
        :param cursor:
        :param extra_fields:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetChatCompletionEventsV1ObservabilityChatCompletionEventsSearchPostRequest(
            page_size=page_size,
            cursor=cursor,
            search_chat_completion_events_request=models.SearchChatCompletionEventsRequest(
                search_params=utils.get_pydantic_model(
                    search_params, models.FilterPayload
                ),
                extra_fields=extra_fields,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/observability/chat-completion-events/search",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.search_chat_completion_events_request,
                False,
                False,
                "json",
                models.SearchChatCompletionEventsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_chat_completion_events_v1_observability_chat_completion_events_search_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SearchChatCompletionEventsResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def search_ids(
        self,
        *,
        search_params: Union[models.FilterPayload, models.FilterPayloadTypedDict],
        extra_fields: OptionalNullable[List[str]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SearchChatCompletionEventIdsResponse:
        r"""Alternative to /search that returns only the IDs and that can return many IDs at once

        :param search_params:
        :param extra_fields:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SearchChatCompletionEventIdsRequest(
            search_params=utils.get_pydantic_model(search_params, models.FilterPayload),
            extra_fields=extra_fields,
        )

        req = self._build_request(
            method="POST",
            path="/v1/observability/chat-completion-events/search-ids",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.SearchChatCompletionEventIdsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_chat_completion_event_ids_v1_observability_chat_completion_events_search_ids_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SearchChatCompletionEventIdsResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def search_ids_async(
        self,
        *,
        search_params: Union[models.FilterPayload, models.FilterPayloadTypedDict],
        extra_fields: OptionalNullable[List[str]] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SearchChatCompletionEventIdsResponse:
        r"""Alternative to /search that returns only the IDs and that can return many IDs at once

        :param search_params:
        :param extra_fields:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.SearchChatCompletionEventIdsRequest(
            search_params=utils.get_pydantic_model(search_params, models.FilterPayload),
            extra_fields=extra_fields,
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/observability/chat-completion-events/search-ids",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request,
                False,
                False,
                "json",
                models.SearchChatCompletionEventIdsRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_chat_completion_event_ids_v1_observability_chat_completion_events_search_ids_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SearchChatCompletionEventIdsResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def fetch(
        self,
        *,
        event_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatCompletionEvent:
        r"""Get Chat Completion Event

        :param event_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetChatCompletionEventV1ObservabilityChatCompletionEventsEventIDGetRequest(
            event_id=event_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/observability/chat-completion-events/{event_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_chat_completion_event_v1_observability_chat_completion_events__event_id__get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ChatCompletionEvent, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def fetch_async(
        self,
        *,
        event_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.ChatCompletionEvent:
        r"""Get Chat Completion Event

        :param event_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetChatCompletionEventV1ObservabilityChatCompletionEventsEventIDGetRequest(
            event_id=event_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/observability/chat-completion-events/{event_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_chat_completion_event_v1_observability_chat_completion_events__event_id__get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.ChatCompletionEvent, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def fetch_similar_events(
        self,
        *,
        event_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SearchChatCompletionEventsResponse:
        r"""Get Similar Chat Completion Events

        :param event_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetSimilarChatCompletionEventsV1ObservabilityChatCompletionEventsEventIDSimilarEventsGetRequest(
            event_id=event_id,
        )

        req = self._build_request(
            method="GET",
            path="/v1/observability/chat-completion-events/{event_id}/similar-events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_similar_chat_completion_events_v1_observability_chat_completion_events__event_id__similar_events_get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SearchChatCompletionEventsResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def fetch_similar_events_async(
        self,
        *,
        event_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.SearchChatCompletionEventsResponse:
        r"""Get Similar Chat Completion Events

        :param event_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetSimilarChatCompletionEventsV1ObservabilityChatCompletionEventsEventIDSimilarEventsGetRequest(
            event_id=event_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/observability/chat-completion-events/{event_id}/similar-events",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get_similar_chat_completion_events_v1_observability_chat_completion_events__event_id__similar_events_get",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                models.SearchChatCompletionEventsResponse, http_res
            )
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    def judge(
        self,
        *,
        event_id: str,
        judge_definition: Union[
            models.CreateJudgeRequest, models.CreateJudgeRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.JudgeOutput:
        r"""Run Judge on an event based on the given options

        :param event_id:
        :param judge_definition:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.JudgeChatCompletionEventV1ObservabilityChatCompletionEventsEventIDLiveJudgingPostRequest(
            event_id=event_id,
            judge_chat_completion_event_request=models.JudgeChatCompletionEventRequest(
                judge_definition=utils.get_pydantic_model(
                    judge_definition, models.CreateJudgeRequest
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/observability/chat-completion-events/{event_id}/live-judging",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.judge_chat_completion_event_request,
                False,
                False,
                "json",
                models.JudgeChatCompletionEventRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="judge_chat_completion_event_v1_observability_chat_completion_events__event_id__live_judging_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.JudgeOutput, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)

    async def judge_async(
        self,
        *,
        event_id: str,
        judge_definition: Union[
            models.CreateJudgeRequest, models.CreateJudgeRequestTypedDict
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.JudgeOutput:
        r"""Run Judge on an event based on the given options

        :param event_id:
        :param judge_definition:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.JudgeChatCompletionEventV1ObservabilityChatCompletionEventsEventIDLiveJudgingPostRequest(
            event_id=event_id,
            judge_chat_completion_event_request=models.JudgeChatCompletionEventRequest(
                judge_definition=utils.get_pydantic_model(
                    judge_definition, models.CreateJudgeRequest
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/observability/chat-completion-events/{event_id}/live-judging",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.judge_chat_completion_event_request,
                False,
                False,
                "json",
                models.JudgeChatCompletionEventRequest,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="judge_chat_completion_event_v1_observability_chat_completion_events__event_id__live_judging_post",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "404", "408", "409", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(models.JudgeOutput, http_res)
        if utils.match_response(
            http_res, ["400", "404", "408", "409", "422"], "application/json"
        ):
            response_data = unmarshal_json_response(
                errors.ObservabilityErrorData, http_res
            )
            raise errors.ObservabilityError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.SDKError("API error occurred", http_res, http_res_text)

        raise errors.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 7531bf461dc2

from __future__ import annotations
from .tempogettraceresponse import TempoGetTraceResponse, TempoGetTraceResponseTypedDict
from .workflowexecutionstatus import WorkflowExecutionStatus
from datetime import datetime
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Any
from typing_extensions import NotRequired, TypedDict


class WorkflowExecutionTraceOTelResponseTypedDict(TypedDict):
    workflow_name: str
    r"""The name of the workflow"""
    execution_id: str
    r"""The ID of the workflow execution"""
    root_execution_id: str
    r"""The root execution ID of the workflow execution"""
    status: Nullable[WorkflowExecutionStatus]
    r"""The status of the workflow execution"""
    start_time: datetime
    r"""The start time of the workflow execution"""
    end_time: Nullable[datetime]
    r"""The end time of the workflow execution, if available"""
    result: Nullable[Any]
    r"""The result of the workflow execution, if available"""
    data_source: str
    r"""The data source of the trace"""
    parent_execution_id: NotRequired[Nullable[str]]
    r"""The parent execution ID of the workflow execution"""
    total_duration_ms: NotRequired[Nullable[int]]
    r"""The total duration of the trace in milliseconds"""
    otel_trace_id: NotRequired[Nullable[str]]
    r"""The ID of the trace"""
    otel_trace_data: NotRequired[Nullable[TempoGetTraceResponseTypedDict]]
    r"""The raw OpenTelemetry trace data"""


class WorkflowExecutionTraceOTelResponse(BaseModel):
    workflow_name: str
    r"""The name of the workflow"""

    execution_id: str
    r"""The ID of the workflow execution"""

    root_execution_id: str
    r"""The root execution ID of the workflow execution"""

    status: Nullable[WorkflowExecutionStatus]
    r"""The status of the workflow execution"""

    start_time: datetime
    r"""The start time of the workflow execution"""

    end_time: Nullable[datetime]
    r"""The end time of the workflow execution, if available"""

    result: Nullable[Any]
    r"""The result of the workflow execution, if available"""

    data_source: str
    r"""The data source of the trace"""

    parent_execution_id: OptionalNullable[str] = UNSET
    r"""The parent execution ID of the workflow execution"""

    total_duration_ms: OptionalNullable[int] = UNSET
    r"""The total duration of the trace in milliseconds"""

    otel_trace_id: OptionalNullable[str] = UNSET
    r"""The ID of the trace"""

    otel_trace_data: OptionalNullable[TempoGetTraceResponse] = UNSET
    r"""The raw OpenTelemetry trace data"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "parent_execution_id",
                "total_duration_ms",
                "otel_trace_id",
                "otel_trace_data",
            ]
        )
        nullable_fields = set(
            [
                "parent_execution_id",
                "status",
                "end_time",
                "total_duration_ms",
                "result",
                "otel_trace_id",
                "otel_trace_data",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: c72b74412547

from __future__ import annotations
from .moderationllmv1config import ModerationLlmv1Config, ModerationLlmv1ConfigTypedDict
from .moderationllmv2config import ModerationLlmv2Config, ModerationLlmv2ConfigTypedDict
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class GuardrailConfigTypedDict(TypedDict):
    block_on_error: NotRequired[bool]
    r"""If true, return HTTP 403 and block request in the event of a server-side error"""
    moderation_llm_v1: NotRequired[Nullable[ModerationLlmv1ConfigTypedDict]]
    moderation_llm_v2: NotRequired[Nullable[ModerationLlmv2ConfigTypedDict]]


class GuardrailConfig(BaseModel):
    block_on_error: Optional[bool] = False
    r"""If true, return HTTP 403 and block request in the event of a server-side error"""

    moderation_llm_v1: OptionalNullable[ModerationLlmv1Config] = UNSET

    moderation_llm_v2: OptionalNullable[ModerationLlmv2Config] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["block_on_error", "moderation_llm_v1", "moderation_llm_v2"]
        )
        nullable_fields = set(["moderation_llm_v1", "moderation_llm_v2"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: cbe8c44aee50

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


ToolType = Union[
    Literal[
        "rag",
        "image",
        "code",
        "event",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 56a16810d5de

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


WorkflowExecutionStatus = Union[
    Literal[
        "RUNNING",
        "COMPLETED",
        "FAILED",
        "CANCELED",
        "TERMINATED",
        "CONTINUED_AS_NEW",
        "TIMED_OUT",
        "RETRYING_AFTER_ERROR",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 0fd96a7c058b

from __future__ import annotations
from .encodedpayloadoptions import EncodedPayloadOptions
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import ConfigDict, model_serializer
from typing import Any, Dict, List, Optional, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


class SignalInvocationBodyNetworkEncodedInputTypedDict(TypedDict):
    b64payload: str
    r"""The encoded payload"""
    encoding_options: NotRequired[List[EncodedPayloadOptions]]
    r"""The encoding of the payload"""
    empty: NotRequired[bool]
    r"""Whether the payload is empty"""


class SignalInvocationBodyNetworkEncodedInput(BaseModel):
    model_config = ConfigDict(
        populate_by_name=True, arbitrary_types_allowed=True, extra="allow"
    )
    __pydantic_extra__: Dict[str, Any] = pydantic.Field(init=False)

    b64payload: str
    r"""The encoded payload"""

    encoding_options: Optional[List[EncodedPayloadOptions]] = None
    r"""The encoding of the payload"""

    empty: Optional[bool] = False
    r"""Whether the payload is empty"""

    @property
    def additional_properties(self):
        return self.__pydantic_extra__

    @additional_properties.setter
    def additional_properties(self, value):
        self.__pydantic_extra__ = value  # pyright: ignore[reportIncompatibleVariableOverride]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["encoding_options", "empty"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            serialized.pop(k, serialized.pop(n, None))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val
        for k, v in serialized.items():
            m[k] = v

        return m


SignalInvocationBodyInputTypedDict = TypeAliasType(
    "SignalInvocationBodyInputTypedDict",
    Union[SignalInvocationBodyNetworkEncodedInputTypedDict, Dict[str, Any]],
)
r"""Input data for the signal, matching its schema"""


SignalInvocationBodyInput = TypeAliasType(
    "SignalInvocationBodyInput",
    Union[SignalInvocationBodyNetworkEncodedInput, Dict[str, Any]],
)
r"""Input data for the signal, matching its schema"""


class SignalInvocationBodyTypedDict(TypedDict):
    name: str
    r"""The name of the signal to send"""
    input: NotRequired[Nullable[SignalInvocationBodyInputTypedDict]]
    r"""Input data for the signal, matching its schema"""


class SignalInvocationBody(BaseModel):
    name: str
    r"""The name of the signal to send"""

    input: OptionalNullable[SignalInvocationBodyInput] = UNSET
    r"""Input data for the signal, matching its schema"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["input"])
        nullable_fields = set(["input"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 1a530d3674d8

from __future__ import annotations
from .metric import Metric, MetricTypedDict
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class CheckpointTypedDict(TypedDict):
    metrics: MetricTypedDict
    r"""Metrics at the step number during the fine-tuning job. Use these metrics to assess if the training is going smoothly (loss should decrease, token accuracy should increase)."""
    step_number: int
    r"""The step number that the checkpoint was created at."""
    created_at: int
    r"""The UNIX timestamp (in seconds) for when the checkpoint was created."""


class Checkpoint(BaseModel):
    metrics: Metric
    r"""Metrics at the step number during the fine-tuning job. Use these metrics to assess if the training is going smoothly (loss should decrease, token accuracy should increase)."""

    step_number: int
    r"""The step number that the checkpoint was created at."""

    created_at: int
    r"""The UNIX timestamp (in seconds) for when the checkpoint was created."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 201353e45f89

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class FieldGroupTypedDict(TypedDict):
    name: str
    label: str


class FieldGroup(BaseModel):
    name: str

    label: str



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 20a423148117

from __future__ import annotations
from .activitytaskcompletedresponse import (
    ActivityTaskCompletedResponse,
    ActivityTaskCompletedResponseTypedDict,
)
from .activitytaskfailedresponse import (
    ActivityTaskFailedResponse,
    ActivityTaskFailedResponseTypedDict,
)
from .activitytaskretryingresponse import (
    ActivityTaskRetryingResponse,
    ActivityTaskRetryingResponseTypedDict,
)
from .activitytaskstartedresponse import (
    ActivityTaskStartedResponse,
    ActivityTaskStartedResponseTypedDict,
)
from .customtaskcanceledresponse import (
    CustomTaskCanceledResponse,
    CustomTaskCanceledResponseTypedDict,
)
from .customtaskcompletedresponse import (
    CustomTaskCompletedResponse,
    CustomTaskCompletedResponseTypedDict,
)
from .customtaskfailedresponse import (
    CustomTaskFailedResponse,
    CustomTaskFailedResponseTypedDict,
)
from .customtaskinprogressresponse import (
    CustomTaskInProgressResponse,
    CustomTaskInProgressResponseTypedDict,
)
from .customtaskstartedresponse import (
    CustomTaskStartedResponse,
    CustomTaskStartedResponseTypedDict,
)
from .customtasktimedoutresponse import (
    CustomTaskTimedOutResponse,
    CustomTaskTimedOutResponseTypedDict,
)
from .workflowexecutioncanceledresponse import (
    WorkflowExecutionCanceledResponse,
    WorkflowExecutionCanceledResponseTypedDict,
)
from .workflowexecutioncompletedresponse import (
    WorkflowExecutionCompletedResponse,
    WorkflowExecutionCompletedResponseTypedDict,
)
from .workflowexecutioncontinuedasnewresponse import (
    WorkflowExecutionContinuedAsNewResponse,
    WorkflowExecutionContinuedAsNewResponseTypedDict,
)
from .workflowexecutionfailedresponse import (
    WorkflowExecutionFailedResponse,
    WorkflowExecutionFailedResponseTypedDict,
)
from .workflowexecutionstartedresponse import (
    WorkflowExecutionStartedResponse,
    WorkflowExecutionStartedResponseTypedDict,
)
from .workflowtaskfailedresponse import (
    WorkflowTaskFailedResponse,
    WorkflowTaskFailedResponseTypedDict,
)
from .workflowtasktimedoutresponse import (
    WorkflowTaskTimedOutResponse,
    WorkflowTaskTimedOutResponseTypedDict,
)
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Union
from typing_extensions import NotRequired, TypeAliasType, TypedDict


ListWorkflowEventResponseEventTypedDict = TypeAliasType(
    "ListWorkflowEventResponseEventTypedDict",
    Union[
        WorkflowExecutionStartedResponseTypedDict,
        WorkflowExecutionCompletedResponseTypedDict,
        WorkflowExecutionFailedResponseTypedDict,
        WorkflowExecutionCanceledResponseTypedDict,
        WorkflowExecutionContinuedAsNewResponseTypedDict,
        WorkflowTaskTimedOutResponseTypedDict,
        WorkflowTaskFailedResponseTypedDict,
        CustomTaskStartedResponseTypedDict,
        CustomTaskInProgressResponseTypedDict,
        CustomTaskCompletedResponseTypedDict,
        CustomTaskFailedResponseTypedDict,
        CustomTaskTimedOutResponseTypedDict,
        CustomTaskCanceledResponseTypedDict,
        ActivityTaskStartedResponseTypedDict,
        ActivityTaskCompletedResponseTypedDict,
        ActivityTaskRetryingResponseTypedDict,
        ActivityTaskFailedResponseTypedDict,
    ],
)


ListWorkflowEventResponseEvent = TypeAliasType(
    "ListWorkflowEventResponseEvent",
    Union[
        WorkflowExecutionStartedResponse,
        WorkflowExecutionCompletedResponse,
        WorkflowExecutionFailedResponse,
        WorkflowExecutionCanceledResponse,
        WorkflowExecutionContinuedAsNewResponse,
        WorkflowTaskTimedOutResponse,
        WorkflowTaskFailedResponse,
        CustomTaskStartedResponse,
        CustomTaskInProgressResponse,
        CustomTaskCompletedResponse,
        CustomTaskFailedResponse,
        CustomTaskTimedOutResponse,
        CustomTaskCanceledResponse,
        ActivityTaskStartedResponse,
        ActivityTaskCompletedResponse,
        ActivityTaskRetryingResponse,
        ActivityTaskFailedResponse,
    ],
)


class ListWorkflowEventResponseTypedDict(TypedDict):
    events: List[ListWorkflowEventResponseEventTypedDict]
    r"""List of workflow events."""
    next_cursor: NotRequired[Nullable[str]]
    r"""Cursor for pagination."""


class ListWorkflowEventResponse(BaseModel):
    events: List[ListWorkflowEventResponseEvent]
    r"""List of workflow events."""

    next_cursor: OptionalNullable[str] = UNSET
    r"""Cursor for pagination."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["next_cursor"])
        nullable_fields = set(["next_cursor"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 95abc4ec799a

from __future__ import annotations
from mistralai.client.types import UnrecognizedStr
from typing import Literal, Union


MistralPromptMode = Union[Literal["reasoning",], UnrecognizedStr]
r"""Available options to the prompt_mode argument on the chat completion endpoint.
Values represent high-level intent. Assignment to actual SPs is handled internally.
System prompt may include knowledge cutoff date, model capabilities, tone to use, safety guidelines, etc.
"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 8c8b08d853f6

from __future__ import annotations
from .agenthandoffdoneevent import AgentHandoffDoneEvent, AgentHandoffDoneEventTypedDict
from .agenthandoffstartedevent import (
    AgentHandoffStartedEvent,
    AgentHandoffStartedEventTypedDict,
)
from .functioncallevent import FunctionCallEvent, FunctionCallEventTypedDict
from .messageoutputevent import MessageOutputEvent, MessageOutputEventTypedDict
from .responsedoneevent import ResponseDoneEvent, ResponseDoneEventTypedDict
from .responseerrorevent import ResponseErrorEvent, ResponseErrorEventTypedDict
from .responsestartedevent import ResponseStartedEvent, ResponseStartedEventTypedDict
from .ssetypes import SSETypes
from .toolexecutiondeltaevent import (
    ToolExecutionDeltaEvent,
    ToolExecutionDeltaEventTypedDict,
)
from .toolexecutiondoneevent import (
    ToolExecutionDoneEvent,
    ToolExecutionDoneEventTypedDict,
)
from .toolexecutionstartedevent import (
    ToolExecutionStartedEvent,
    ToolExecutionStartedEventTypedDict,
)
from functools import partial
from mistralai.client.types import BaseModel
from mistralai.client.utils.unions import parse_open_union
from pydantic import ConfigDict
from pydantic.functional_validators import BeforeValidator
from typing import Any, Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


ConversationEventsDataTypedDict = TypeAliasType(
    "ConversationEventsDataTypedDict",
    Union[
        ResponseStartedEventTypedDict,
        ResponseDoneEventTypedDict,
        ResponseErrorEventTypedDict,
        ToolExecutionDeltaEventTypedDict,
        ToolExecutionDoneEventTypedDict,
        AgentHandoffStartedEventTypedDict,
        AgentHandoffDoneEventTypedDict,
        ToolExecutionStartedEventTypedDict,
        MessageOutputEventTypedDict,
        FunctionCallEventTypedDict,
    ],
)


class UnknownConversationEventsData(BaseModel):
    r"""A ConversationEventsData variant the SDK doesn't recognize. Preserves the raw payload."""

    type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_CONVERSATION_EVENTS_DATA_VARIANTS: dict[str, Any] = {
    "agent.handoff.done": AgentHandoffDoneEvent,
    "agent.handoff.started": AgentHandoffStartedEvent,
    "conversation.response.done": ResponseDoneEvent,
    "conversation.response.error": ResponseErrorEvent,
    "conversation.response.started": ResponseStartedEvent,
    "function.call.delta": FunctionCallEvent,
    "message.output.delta": MessageOutputEvent,
    "tool.execution.delta": ToolExecutionDeltaEvent,
    "tool.execution.done": ToolExecutionDoneEvent,
    "tool.execution.started": ToolExecutionStartedEvent,
}


ConversationEventsData = Annotated[
    Union[
        AgentHandoffDoneEvent,
        AgentHandoffStartedEvent,
        ResponseDoneEvent,
        ResponseErrorEvent,
        ResponseStartedEvent,
        FunctionCallEvent,
        MessageOutputEvent,
        ToolExecutionDeltaEvent,
        ToolExecutionDoneEvent,
        ToolExecutionStartedEvent,
        UnknownConversationEventsData,
    ],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="type",
            variants=_CONVERSATION_EVENTS_DATA_VARIANTS,
            unknown_cls=UnknownConversationEventsData,
            union_name="ConversationEventsData",
        )
    ),
]


class ConversationEventsTypedDict(TypedDict):
    event: SSETypes
    r"""Server side events sent when streaming a conversation response."""
    data: ConversationEventsDataTypedDict


class ConversationEvents(BaseModel):
    event: SSETypes
    r"""Server side events sent when streaming a conversation response."""

    data: ConversationEventsData



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: c530f2fc64d0

from __future__ import annotations
from .agentconversation import AgentConversation, AgentConversationTypedDict
from .modelconversation import ModelConversation, ModelConversationTypedDict
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class AgentsAPIV1ConversationsGetRequestTypedDict(TypedDict):
    conversation_id: str
    r"""ID of the conversation from which we are fetching metadata."""


class AgentsAPIV1ConversationsGetRequest(BaseModel):
    conversation_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""ID of the conversation from which we are fetching metadata."""


ResponseV1ConversationsGetTypedDict = TypeAliasType(
    "ResponseV1ConversationsGetTypedDict",
    Union[AgentConversationTypedDict, ModelConversationTypedDict],
)
r"""Successful Response"""


ResponseV1ConversationsGet = TypeAliasType(
    "ResponseV1ConversationsGet", Union[AgentConversation, ModelConversation]
)
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from mistralai.gcp.client.types import BaseModel
from mistralai.gcp.client.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class TextChunkTypedDict(TypedDict):
    text: str
    type: Literal["text"]


class TextChunk(BaseModel):
    text: str

    type: Annotated[
        Annotated[Literal["text"], AfterValidator(validate_const("text"))],
        pydantic.Field(alias="type"),
    ] = "text"


try:
    TextChunk.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f55510f5ff82

from __future__ import annotations
from mistralai.client.types import BaseModel
from mistralai.client.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class APIKeyAuthTypedDict(TypedDict):
    value: str
    type: Literal["api-key"]


class APIKeyAuth(BaseModel):
    value: str

    type: Annotated[
        Annotated[Literal["api-key"], AfterValidator(validate_const("api-key"))],
        pydantic.Field(alias="type"),
    ] = "api-key"


try:
    APIKeyAuth.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: e13f9009902b

from __future__ import annotations
from .messageinputentry import MessageInputEntry, MessageInputEntryTypedDict
from .messageoutputentry import MessageOutputEntry, MessageOutputEntryTypedDict
from typing import Union
from typing_extensions import TypeAliasType


MessageEntriesTypedDict = TypeAliasType(
    "MessageEntriesTypedDict",
    Union[MessageInputEntryTypedDict, MessageOutputEntryTypedDict],
)


MessageEntries = TypeAliasType(
    "MessageEntries", Union[MessageInputEntry, MessageOutputEntry]
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 5da876c66fc5

from __future__ import annotations
from .workflowexecutionrequest import (
    WorkflowExecutionRequest,
    WorkflowExecutionRequestTypedDict,
)
from .workflowexecutionresponse import (
    WorkflowExecutionResponse,
    WorkflowExecutionResponseTypedDict,
)
from .workflowexecutionsyncresponse import (
    WorkflowExecutionSyncResponse,
    WorkflowExecutionSyncResponseTypedDict,
)
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing import Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class ExecuteWorkflowV1WorkflowsWorkflowIdentifierExecutePostRequestTypedDict(
    TypedDict
):
    workflow_identifier: str
    workflow_execution_request: WorkflowExecutionRequestTypedDict


class ExecuteWorkflowV1WorkflowsWorkflowIdentifierExecutePostRequest(BaseModel):
    workflow_identifier: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    workflow_execution_request: Annotated[
        WorkflowExecutionRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


ResponseExecuteWorkflowV1WorkflowsWorkflowIdentifierExecutePostTypedDict = (
    TypeAliasType(
        "ResponseExecuteWorkflowV1WorkflowsWorkflowIdentifierExecutePostTypedDict",
        Union[
            WorkflowExecutionSyncResponseTypedDict, WorkflowExecutionResponseTypedDict
        ],
    )
)
r"""Successful Response"""


ResponseExecuteWorkflowV1WorkflowsWorkflowIdentifierExecutePost = TypeAliasType(
    "ResponseExecuteWorkflowV1WorkflowsWorkflowIdentifierExecutePost",
    Union[WorkflowExecutionSyncResponse, WorkflowExecutionResponse],
)
r"""Successful Response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6d9dc624aafd

from __future__ import annotations
from .classifierfinetunedmodel import (
    ClassifierFineTunedModel,
    ClassifierFineTunedModelTypedDict,
)
from .completionfinetunedmodel import (
    CompletionFineTunedModel,
    CompletionFineTunedModelTypedDict,
)
from .updatemodelrequest import UpdateModelRequest, UpdateModelRequestTypedDict
from functools import partial
from mistralai.client.types import BaseModel
from mistralai.client.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from mistralai.client.utils.unions import parse_open_union
from pydantic import ConfigDict
from pydantic.functional_validators import BeforeValidator
from typing import Any, Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


class JobsAPIRoutesFineTuningUpdateFineTunedModelRequestTypedDict(TypedDict):
    model_id: str
    r"""The ID of the model to update."""
    update_model_request: UpdateModelRequestTypedDict


class JobsAPIRoutesFineTuningUpdateFineTunedModelRequest(BaseModel):
    model_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID of the model to update."""

    update_model_request: Annotated[
        UpdateModelRequest,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]


JobsAPIRoutesFineTuningUpdateFineTunedModelResponseTypedDict = TypeAliasType(
    "JobsAPIRoutesFineTuningUpdateFineTunedModelResponseTypedDict",
    Union[CompletionFineTunedModelTypedDict, ClassifierFineTunedModelTypedDict],
)
r"""OK"""


class UnknownJobsAPIRoutesFineTuningUpdateFineTunedModelResponse(BaseModel):
    r"""A JobsAPIRoutesFineTuningUpdateFineTunedModelResponse variant the SDK doesn't recognize. Preserves the raw payload."""

    model_type: Literal["UNKNOWN"] = "UNKNOWN"
    raw: Any
    is_unknown: Literal[True] = True

    model_config = ConfigDict(frozen=True)


_JOBS_API_ROUTES_FINE_TUNING_UPDATE_FINE_TUNED_MODEL_RESPONSE_VARIANTS: dict[
    str, Any
] = {
    "classifier": ClassifierFineTunedModel,
    "completion": CompletionFineTunedModel,
}


JobsAPIRoutesFineTuningUpdateFineTunedModelResponse = Annotated[
    Union[
        ClassifierFineTunedModel,
        CompletionFineTunedModel,
        UnknownJobsAPIRoutesFineTuningUpdateFineTunedModelResponse,
    ],
    BeforeValidator(
        partial(
            parse_open_union,
            disc_key="model_type",
            variants=_JOBS_API_ROUTES_FINE_TUNING_UPDATE_FINE_TUNED_MODEL_RESPONSE_VARIANTS,
            unknown_cls=UnknownJobsAPIRoutesFineTuningUpdateFineTunedModelResponse,
            union_name="JobsAPIRoutesFineTuningUpdateFineTunedModelResponse",
        )
    ),
]
r"""OK"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: c77172c4a9f6

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class WorkflowScheduleResponseTypedDict(TypedDict):
    schedule_id: str
    r"""The ID of the schedule"""


class WorkflowScheduleResponse(BaseModel):
    schedule_id: str
    r"""The ID of the schedule"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 270800e2c264

from __future__ import annotations
from mistralai.client.types import BaseModel, UNSET_SENTINEL
from mistralai.client.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetCampaignSelectedEventsV1ObservabilityCampaignsCampaignIDSelectedEventsGetRequestTypedDict(
    TypedDict
):
    campaign_id: str
    page_size: NotRequired[int]
    page: NotRequired[int]


class GetCampaignSelectedEventsV1ObservabilityCampaignsCampaignIDSelectedEventsGetRequest(
    BaseModel
):
    campaign_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 50

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = 1

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["page_size", "page"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b2882fa57029

from __future__ import annotations
from mistralai.client.types import BaseModel
from typing_extensions import TypedDict


class ImportDatasetFromFileRequestTypedDict(TypedDict):
    file_id: str


class ImportDatasetFromFileRequest(BaseModel):
    file_id: str



"""Streaming response helpers for OTEL tracing.

Pure functions that parse SSE byte streams and accumulate CompletionChunk
deltas into a ChatCompletionResponse-shaped dict suitable for span enrichment.

TODO: supports chat and agent completion streaming endpoints. Evolutions will
be necessary to support other streaming endpoints (e.g. conversations).

NOTE: The SSE bytes are re-parsed here even though EventStream already
parsed them during iteration.
TracedResponse sits below EventStream and can only accumulate raw bytes; it
has no access to the decoded events. Hooking into EventStream could eliminate
this double-parse, but EventStream is Speakeasy-generated code.
"""

from typing import Any

from mistralai.client.models import CompletionChunk, UsageInfo


def parse_sse_chunks(raw_sse_bytes: bytes) -> list[CompletionChunk]:
    """Parse raw SSE bytes into a list of typed CompletionChunk models.

    Only CompletionChunk is handled.  If new SSE-streamed response types
    are added, parsing and typing here will need updating.
    """
    chunks: list[CompletionChunk] = []
    text = raw_sse_bytes.decode("utf-8", errors="replace")
    for line in text.split("\n"):
        line = line.strip()
        if not line.startswith("data: "):
            continue
        payload = line[6:]
        if payload == "[DONE]":
            continue
        try:
            chunks.append(CompletionChunk.model_validate_json(payload))
        except Exception:
            continue
    return chunks


def accumulate_chunks_to_response_dict(
    chunks: list[CompletionChunk],
) -> dict[str, Any]:
    """Accumulate streaming CompletionChunk deltas into a ChatCompletionResponse-shaped dict."""
    response_id: str | None = None
    model: str | None = None
    usage: UsageInfo | None = None
    choices: dict[int, dict[str, Any]] = {}

    for chunk in chunks:
        response_id = response_id or chunk.id
        model = model or chunk.model
        usage = usage or chunk.usage

        for choice in chunk.choices:
            accumulated = choices.setdefault(
                choice.index,
                {
                    "message": {"role": "assistant", "content": ""},
                    "finish_reason": "",
                },
            )
            msg = accumulated["message"]
            delta = choice.delta
            if isinstance(delta.role, str):
                msg["role"] = delta.role
            if isinstance(delta.content, str) and delta.content:
                msg["content"] += delta.content
            if isinstance(choice.finish_reason, str):
                accumulated["finish_reason"] = choice.finish_reason
            if isinstance(delta.tool_calls, list):
                tc_list = msg.setdefault("tool_calls", [])
                for tc in delta.tool_calls:
                    tc_idx = tc.index if tc.index is not None else len(tc_list)
                    while len(tc_list) <= tc_idx:
                        tc_list.append(
                            {"id": None, "function": {"name": "", "arguments": ""}}
                        )
                    # ToolCall.id defaults to the string "null" (Speakeasy codegen quirk)
                    if tc.id is not None and tc.id != "null":
                        tc_list[tc_idx]["id"] = tc.id
                    if tc.function.name:
                        tc_list[tc_idx]["function"]["name"] += tc.function.name
                    if isinstance(tc.function.arguments, str) and tc.function.arguments:
                        tc_list[tc_idx]["function"]["arguments"] += (
                            tc.function.arguments
                        )

    result: dict[str, Any] = {
        "id": response_id,
        "model": model,
        "choices": [choices[idx] for idx in sorted(choices)],
    }
    if usage is not None:
        result["usage"] = usage.model_dump(mode="json", by_alias=True)
    return result



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 806340497ed4

from __future__ import annotations
from .networkencodedinput import NetworkEncodedInput, NetworkEncodedInputTypedDict
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class WorkflowExecutionRequestTypedDict(TypedDict):
    execution_id: NotRequired[Nullable[str]]
    r"""Allows you to specify a custom execution ID. If not provided, a random ID will be generated."""
    input: NotRequired[Nullable[Dict[str, Any]]]
    r"""The input to the workflow. This should be a dictionary that matches the workflow's input schema."""
    encoded_input: NotRequired[Nullable[NetworkEncodedInputTypedDict]]
    r"""Encoded input to the workflow, used when payload encoding is enabled."""
    wait_for_result: NotRequired[bool]
    r"""If true, wait for the workflow to complete and return the result directly."""
    timeout_seconds: NotRequired[Nullable[float]]
    r"""Maximum time to wait for completion when wait_for_result is true."""
    custom_tracing_attributes: NotRequired[Nullable[Dict[str, str]]]
    task_queue: NotRequired[Nullable[str]]
    r"""Deprecated. Use deployment_name instead."""
    deployment_name: NotRequired[Nullable[str]]
    r"""Name of the deployment to route this execution to"""


class WorkflowExecutionRequest(BaseModel):
    execution_id: OptionalNullable[str] = UNSET
    r"""Allows you to specify a custom execution ID. If not provided, a random ID will be generated."""

    input: OptionalNullable[Dict[str, Any]] = UNSET
    r"""The input to the workflow. This should be a dictionary that matches the workflow's input schema."""

    encoded_input: OptionalNullable[NetworkEncodedInput] = UNSET
    r"""Encoded input to the workflow, used when payload encoding is enabled."""

    wait_for_result: Optional[bool] = False
    r"""If true, wait for the workflow to complete and return the result directly."""

    timeout_seconds: OptionalNullable[float] = UNSET
    r"""Maximum time to wait for completion when wait_for_result is true."""

    custom_tracing_attributes: OptionalNullable[Dict[str, str]] = UNSET

    task_queue: Annotated[
        OptionalNullable[str],
        pydantic.Field(
            deprecated="warning: ** DEPRECATED ** - This will be removed in a future release, please migrate away from it as soon as possible."
        ),
    ] = UNSET
    r"""Deprecated. Use deployment_name instead."""

    deployment_name: OptionalNullable[str] = UNSET
    r"""Name of the deployment to route this execution to"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "execution_id",
                "input",
                "encoded_input",
                "wait_for_result",
                "timeout_seconds",
                "custom_tracing_attributes",
                "task_queue",
                "deployment_name",
            ]
        )
        nullable_fields = set(
            [
                "execution_id",
                "input",
                "encoded_input",
                "timeout_seconds",
                "custom_tracing_attributes",
                "task_queue",
                "deployment_name",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 85cd28932cc7

from __future__ import annotations
from .batcherror import BatchError, BatchErrorTypedDict
from .batchjobstatus import BatchJobStatus
from mistralai.client.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from mistralai.client.utils import validate_const
import pydantic
from pydantic import model_serializer
from pydantic.functional_validators import AfterValidator
from typing import Any, Dict, List, Literal, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class BatchJobTypedDict(TypedDict):
    id: str
    input_files: List[str]
    endpoint: str
    errors: List[BatchErrorTypedDict]
    status: BatchJobStatus
    created_at: int
    total_requests: int
    completed_requests: int
    succeeded_requests: int
    failed_requests: int
    object: Literal["batch"]
    metadata: NotRequired[Nullable[Dict[str, Any]]]
    model: NotRequired[Nullable[str]]
    agent_id: NotRequired[Nullable[str]]
    output_file: NotRequired[Nullable[str]]
    error_file: NotRequired[Nullable[str]]
    outputs: NotRequired[Nullable[List[Dict[str, Any]]]]
    started_at: NotRequired[Nullable[int]]
    completed_at: NotRequired[Nullable[int]]


class BatchJob(BaseModel):
    id: str

    input_files: List[str]

    endpoint: str

    errors: List[BatchError]

    status: BatchJobStatus

    created_at: int

    total_requests: int

    completed_requests: int

    succeeded_requests: int

    failed_requests: int

    object: Annotated[
        Annotated[Optional[Literal["batch"]], AfterValidator(validate_const("batch"))],
        pydantic.Field(alias="object"),
    ] = "batch"

    metadata: OptionalNullable[Dict[str, Any]] = UNSET

    model: OptionalNullable[str] = UNSET

    agent_id: OptionalNullable[str] = UNSET

    output_file: OptionalNullable[str] = UNSET

    error_file: OptionalNullable[str] = UNSET

    outputs: OptionalNullable[List[Dict[str, Any]]] = UNSET

    started_at: OptionalNullable[int] = UNSET

    completed_at: OptionalNullable[int] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "object",
                "metadata",
                "model",
                "agent_id",
                "output_file",
                "error_file",
                "outputs",
                "started_at",
                "completed_at",
            ]
        )
        nullable_fields = set(
            [
                "metadata",
                "model",
                "agent_id",
                "output_file",
                "error_file",
                "outputs",
                "started_at",
                "completed_at",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    BatchJob.model_rebuild()
except NameError:
    pass
